@@ -30,17 +30,15 @@ struct _CogShellClass {
     void           (*shutdown)    (CogShell*);
 };
 
-
-CogShell         *cog_shell_new                     (const char        *name);
-const char       *cog_shell_get_name                (CogShell          *shell);
-WebKitWebContext *cog_shell_get_web_context         (CogShell          *shell);
+CogShell *        cog_shell_new(const char *name, gboolean automated);
+const char *      cog_shell_get_name(CogShell *shell);
+WebKitWebContext *cog_shell_get_web_context(CogShell *shell);
 WebKitSettings   *cog_shell_get_web_settings        (CogShell          *shell);
 WebKitWebView    *cog_shell_get_web_view            (CogShell          *shell);
 GKeyFile         *cog_shell_get_config_file         (CogShell          *shell);
 gdouble           cog_shell_get_device_scale_factor (CogShell          *shell);
-void              cog_shell_set_request_handler     (CogShell          *shell,
-                                                     const char        *scheme,
-                                                     CogRequestHandler *handler);
+gboolean          cog_shell_is_automated(CogShell *shell);
+void              cog_shell_set_request_handler(CogShell *shell, const char *scheme, CogRequestHandler *handler);
 
 void              cog_shell_startup                 (CogShell          *shell);
 void              cog_shell_shutdown                (CogShell          *shell);
