@@ -72,11 +72,14 @@ cog_handle_web_view_load_failed (WebKitWebView  *web_view,
                                  GError         *error,
                                  void           *userdata)
 {
-    // If the resource is going to be shown by a plug-in (or a
-    // media engine) just return FALSE and let WebKit handle it.
-    if (g_error_matches (error,
-                         WEBKIT_PLUGIN_ERROR,
-                         WEBKIT_PLUGIN_ERROR_WILL_HANDLE_LOAD))
+    // If the resource is going to be shown by a plug-in or a
+    // media engine (for the case of API WPE2) just return FALSE and
+    // let WebKit handle it.
+#if COG_USE_WPE2
+    if (g_error_matches(error, WEBKIT_MEDIA_ERROR, WEBKIT_MEDIA_ERROR_WILL_HANDLE_LOAD))
+#else
+    if (g_error_matches(error, WEBKIT_PLUGIN_ERROR, WEBKIT_PLUGIN_ERROR_WILL_HANDLE_LOAD))
+#endif
         return FALSE;
 
     // Ignore cancellation errors as the active URI may be changing
@@ -480,7 +483,7 @@ cog_web_view_connect_default_progress_handlers (WebKitWebView *web_view)
  * @group: Name of a group from the key file.
  * @error: (out) (nullable): Location where to store an error, if any.
  *
- * Reads values from a given `group` of a [class@GLib.KeyFile] object,
+ * Reads values from a given `group` of a [struct@GLib.KeyFile] object,
  * and uses them to set the writable properties of a [class@WebKit.Settings]
  * object.
  *
