@@ -53,7 +53,9 @@ static struct {
     } on_failure;
     char    *web_extensions_dir;
     gboolean ignore_tls_errors;
+#if !COG_USE_WPE2
     gboolean enable_sandbox;
+#endif
     gboolean automation;
 #if HAVE_WEBKIT_NETWORK_PROXY_API
     char    *proxy;
@@ -103,13 +105,16 @@ enum {
 
 struct _CogLauncher {
     GApplication parent;
-    CogPlatform *platform;
     CogShell    *shell;
     gboolean     allow_all_requests;
     gboolean     automated;
 
     WebKitSettings           *web_settings;
+#if COG_USE_WPE2
+    WebKitNetworkSession *network_session;
+#else
     WebKitWebsiteDataManager *web_data_manager;
+#endif
 
 #if COG_HAVE_MEM_PRESSURE
     WebKitMemoryPressureSettings *web_mem_settings;
@@ -206,8 +211,8 @@ cog_launcher_open(GApplication *application, GFile **files, int n_files, const c
     webkit_web_view_load_uri(cog_shell_get_web_view(COG_LAUNCHER(application)->shell), uri);
 }
 
-static gboolean
-platform_setup(CogLauncher *self)
+static CogPlatform *
+platform_setup_once(CogLauncher *self)
 {
     /*
      * Here we resolve the CogPlatform we are going to use. A Cog platform
@@ -216,25 +221,39 @@ platform_setup(CogLauncher *self)
      * a given platform.
      */
 
-    g_debug("%s: Platform name: %s", __func__, s_options.platform_name);
+    /* The documentation of g_getenv() says that the returned string may be overwritten by the next
+     * call to g_getenv(), g_setenv() or g_unsetenv(), so avoid passing it directly as parameter to
+     * other functions that may call any of the *env() functions and then try to use the parameter. */
+    g_autofree const char *platform_name = g_strdup(s_options.platform_name ?: g_getenv("COG_PLATFORM_NAME"));
+    g_clear_pointer(&s_options.platform_name, g_free);
+    g_debug("%s: Platform name: %s", __func__, platform_name);
 
-    g_autoptr(GError)      error = NULL;
-    g_autoptr(CogPlatform) platform = cog_platform_new(s_options.platform_name, &error);
+    g_autoptr(GError) error = NULL;
+    CogPlatform      *platform = cog_platform_new(platform_name, &error);
     if (!platform) {
         g_warning("Cannot create platform: %s", error->message);
-        return FALSE;
+        return NULL;
     }
-    g_clear_pointer(&s_options.platform_name, g_free);
 
-    if (!cog_platform_setup(platform, self->shell, s_options.platform_params ?: "", &error)) {
+    g_autofree const char *platform_params = g_strdup(s_options.platform_params ?: g_getenv("COG_PLATFORM_PARAMS"));
+    g_clear_pointer(&s_options.platform_params, g_free);
+    g_debug("%s: Platform params: %s", __func__, platform_params);
+
+    if (!cog_platform_setup(platform, self->shell, platform_params ?: "", &error)) {
         g_warning("Platform setup failed: %s", error->message);
-        return FALSE;
+        return NULL;
     }
 
-    self->platform = g_steal_pointer(&platform);
+    g_debug("%s: Selected %s @ %p", __func__, G_OBJECT_TYPE_NAME(platform), platform);
+    return platform;
+}
 
-    g_debug("%s: Selected %s @ %p", __func__, G_OBJECT_TYPE_NAME(self->platform), self->platform);
-    return TRUE;
+static CogPlatform *
+platform_setup(CogLauncher *self)
+{
+    static GOnce s_once = G_ONCE_INIT;
+    g_once(&s_once, (GThreadFunc) platform_setup_once, self);
+    return s_once.retval;
 }
 
 static void *
@@ -258,9 +277,10 @@ cog_launcher_create_view(CogLauncher *self, CogShell *shell)
     WebKitWebViewBackend *view_backend = NULL;
 
     // Try to load the platform plug-in specified in the command line.
-    if (platform_setup(self)) {
+    CogPlatform *platform = platform_setup(self);
+    if (platform) {
         g_autoptr(GError) error = NULL;
-        view_backend = cog_platform_get_view_backend(self->platform, NULL, &error);
+        view_backend = cog_platform_get_view_backend(platform, NULL, &error);
         if (!view_backend) {
             g_assert(error);
             g_warning("Failed to get platform's view backend: %s", error->message);
@@ -280,6 +300,9 @@ cog_launcher_create_view(CogLauncher *self, CogShell *shell)
 
     g_autoptr(WebKitWebView) web_view =
         g_object_new(WEBKIT_TYPE_WEB_VIEW, "settings", cog_shell_get_web_settings(shell), "web-context", web_context,
+#if COG_USE_WPE2
+                     "network-session", self->network_session,
+#endif
                      "zoom-level", s_options.scale_factor, "backend", view_backend, "is-controlled-by-automation",
                      cog_shell_is_automated(shell), NULL);
 
@@ -291,9 +314,11 @@ cog_launcher_create_view(CogLauncher *self, CogShell *shell)
 
     g_signal_connect(web_view, "create", G_CALLBACK(on_web_view_create), NULL);
 
-    cog_platform_init_web_view(self->platform, web_view);
-    g_autoptr(WebKitInputMethodContext) im_context = cog_platform_create_im_context(self->platform);
-    webkit_web_view_set_input_method_context(web_view, im_context);
+    if (platform) {
+        cog_platform_init_web_view(platform, web_view);
+        g_autoptr(WebKitInputMethodContext) im_context = cog_platform_create_im_context(platform);
+        webkit_web_view_set_input_method_context(web_view, im_context);
+    }
 
     if (s_options.background_color != NULL) {
         WebKitColor color;
@@ -304,6 +329,7 @@ cog_launcher_create_view(CogLauncher *self, CogShell *shell)
         else
             g_error("'%s' doesn't represent a valid #RRGGBBAA or CSS color format.", s_options.background_color);
     }
+    g_clear_pointer(&s_options.background_color, g_free);
 
     switch (s_options.on_failure.action_id) {
     case WEBPROCESS_FAIL_ERROR_PAGE:
@@ -349,20 +375,29 @@ cog_launcher_startup(GApplication *application)
     g_application_hold(application);
 
     CogLauncher *self = COG_LAUNCHER(application);
-    self->shell =
-        g_object_new(COG_TYPE_SHELL, "name", g_get_prgname(), "automated", self->automated, "web-settings",
-                     self->web_settings, "web-data-manager", self->web_data_manager,
+    self->shell = g_object_new(
+        COG_TYPE_SHELL, "name", g_get_prgname(), "automated", self->automated, "web-settings", self->web_settings,
+#if !COG_USE_WPE2
+        "web-data-manager", self->web_data_manager,
+#endif
 #if COG_HAVE_MEM_PRESSURE
-                     "web-memory-settings", self->web_mem_settings, "network-memory-settings", self->net_mem_settings,
+        "web-memory-settings", self->web_mem_settings, "network-memory-settings", self->net_mem_settings,
 #endif /* COG_HAVE_MEM_PRESSURE */
-                     NULL);
+        NULL);
     g_signal_connect_swapped(self->shell, "create-view", G_CALLBACK(cog_launcher_create_view), self);
     g_signal_connect(self->shell, "notify::web-view", G_CALLBACK(on_notify_web_view), self);
 
+#if COG_USE_WPE2
+    if (s_options.web_extensions_dir)
+        webkit_web_context_set_web_process_extensions_directory(cog_shell_get_web_context(self->shell),
+                                                                s_options.web_extensions_dir);
+#else
     if (s_options.web_extensions_dir)
         webkit_web_context_set_web_extensions_directory(cog_shell_get_web_context(self->shell),
                                                         s_options.web_extensions_dir);
     webkit_web_context_set_sandbox_enabled(cog_shell_get_web_context(self->shell), s_options.enable_sandbox);
+#endif
+    g_clear_pointer(&s_options.web_extensions_dir, g_free);
 
     g_object_set(self->shell, "device-scale-factor", s_options.device_scale_factor, NULL);
 
@@ -384,7 +419,13 @@ cog_launcher_startup(GApplication *application)
         g_object_set(self->shell, "config-file", key_file, NULL);
     }
 
-#if WEBKIT_CHECK_VERSION(2, 32, 0)
+#if COG_USE_WPE2
+    if (!self->automated) {
+        webkit_network_session_set_tls_errors_policy(self->network_session,
+                                                     s_options.ignore_tls_errors ? WEBKIT_TLS_ERRORS_POLICY_IGNORE
+                                                                                 : WEBKIT_TLS_ERRORS_POLICY_FAIL);
+    }
+#elif WEBKIT_CHECK_VERSION(2, 32, 0)
     webkit_website_data_manager_set_tls_errors_policy(cog_launcher_get_web_data_manager(self),
                                                       s_options.ignore_tls_errors ? WEBKIT_TLS_ERRORS_POLICY_IGNORE
                                                                                   : WEBKIT_TLS_ERRORS_POLICY_FAIL);
@@ -395,6 +436,14 @@ cog_launcher_startup(GApplication *application)
 #endif
 }
 
+static void
+cog_launcher_activate(GApplication *application)
+{
+    /* GApplication warns if activate is not handled. Usually this signal will focus a
+       window but this doesn't apply to many of our platforms so this is a noop. */
+    G_APPLICATION_CLASS(cog_launcher_parent_class)->activate(application);
+}
+
 static void
 cog_launcher_shutdown(GApplication *application)
 {
@@ -414,18 +463,24 @@ cog_launcher_dispose(GObject *object)
         g_clear_object(&launcher->shell);
     }
 
-    g_clear_object(&launcher->platform);
-
     g_clear_handle_id(&launcher->sigint_source, g_source_remove);
     g_clear_handle_id(&launcher->sigterm_source, g_source_remove);
 
     g_clear_object(&launcher->web_settings);
 
+#if COG_USE_WPE2
+    g_clear_object(&launcher->network_session);
+#else
+    g_clear_object(&launcher->web_data_manager);
+#endif
+
 #if COG_HAVE_MEM_PRESSURE
     g_clear_pointer(&launcher->web_mem_settings, webkit_memory_pressure_settings_free);
     g_clear_pointer(&launcher->net_mem_settings, webkit_memory_pressure_settings_free);
 #endif /* COG_HAVE_MEM_PRESSURE */
 
+    g_object_unref(cog_platform_get_default());
+
     G_OBJECT_CLASS(cog_launcher_parent_class)->dispose(object);
 }
 
@@ -475,6 +530,16 @@ option_entry_parse_cookie_jar(const char *option G_GNUC_UNUSED,
         g_assert_not_reached();
     }
 
+#if COG_USE_WPE2
+    if (launcher->automated) {
+        g_set_error_literal(error,
+                            G_OPTION_ERROR,
+                            G_OPTION_ERROR_BAD_VALUE,
+                            "Cannot set persistent cookies in automation mode");
+        return FALSE;
+    }
+#endif
+
     g_autofree char *cookie_jar_path = NULL;
     g_autofree char *format_name = g_strdup(value);
     char            *path = strchr(format_name, ':');
@@ -513,12 +578,16 @@ option_entry_parse_cookie_jar(const char *option G_GNUC_UNUSED,
 
     if (!cookie_jar_path) {
         g_autofree char *file_name = g_strconcat("cookies.", format_name, NULL);
-        cookie_jar_path = g_build_filename(
-            webkit_website_data_manager_get_base_data_directory(launcher->web_data_manager), file_name, NULL);
+        cookie_jar_path = g_build_filename(g_get_user_data_dir(), g_get_prgname(), file_name, NULL);
     }
 
     webkit_cookie_manager_set_persistent_storage(
-        webkit_website_data_manager_get_cookie_manager(launcher->web_data_manager), cookie_jar_path, enum_value->value);
+#if COG_USE_WPE2
+        webkit_network_session_get_cookie_manager(launcher->network_session),
+#else
+        webkit_website_data_manager_get_cookie_manager(launcher->web_data_manager),
+#endif
+        cookie_jar_path, enum_value->value);
     return TRUE;
 }
 
@@ -553,10 +622,16 @@ option_entry_parse_cookie_store(const char *option G_GNUC_UNUSED,
                                 CogLauncher       *launcher,
                                 GError           **error)
 {
+#if COG_USE_WPE2
+    WebKitCookieManager *cookie_manager =
+        launcher->network_session ? webkit_network_session_get_cookie_manager(launcher->network_session) : NULL;
+#else
     WebKitCookieManager *cookie_manager = webkit_website_data_manager_get_cookie_manager(launcher->web_data_manager);
+#endif
 
     if (strcmp(value, "help") == 0) {
-        const WebKitCookieAcceptPolicy default_mode = cookie_manager_get_accept_policy(cookie_manager);
+        const WebKitCookieAcceptPolicy default_mode = cookie_manager ? cookie_manager_get_accept_policy(cookie_manager)
+                                                                     : WEBKIT_COOKIE_POLICY_ACCEPT_NO_THIRD_PARTY;
         g_autoptr(GEnumClass)          enum_class = g_type_class_ref(WEBKIT_TYPE_COOKIE_ACCEPT_POLICY);
         for (unsigned i = 0; i < enum_class->n_values; i++) {
             const char *format = (enum_class->values[i].value == default_mode) ? "%s (default)\n" : "%s\n";
@@ -566,6 +641,14 @@ option_entry_parse_cookie_store(const char *option G_GNUC_UNUSED,
         g_assert_not_reached();
     }
 
+#if COG_USE_WPE2
+    if (!cookie_manager) {
+        g_set_error_literal(error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE,
+                            "Cannot set cookie storing mode in automation mode");
+        return FALSE;
+    }
+#endif
+
     const GEnumValue *enum_value = cog_g_enum_get_value(WEBKIT_TYPE_COOKIE_ACCEPT_POLICY, value);
     if (!enum_value) {
         g_set_error(error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE, "Invalid cookie storing mode '%s'", value);
@@ -620,6 +703,12 @@ option_entry_parse_cookie_add(const char *option G_GNUC_UNUSED,
                               CogLauncher       *launcher,
                               GError **error     G_GNUC_UNUSED)
 {
+#if COG_USE_WPE2
+    if (launcher->automated) {
+        g_set_error_literal(error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE, "Cannot add cookies in automation mode");
+        return FALSE;
+    }
+#endif
     g_autoptr(GMainLoop)  loop = NULL;
     g_autoptr(SoupCookie) cookie = NULL;
     g_autofree char      *domain = g_strdup(value);
@@ -681,7 +770,12 @@ option_entry_parse_cookie_add(const char *option G_GNUC_UNUSED,
     // Adding a cookie is an asynchronous operation, so spin up an
     // event loop until to block until the operation completes.
     loop = g_main_loop_new(NULL, FALSE);
-    webkit_cookie_manager_add_cookie(webkit_website_data_manager_get_cookie_manager(launcher->web_data_manager),
+#if COG_USE_WPE2
+    WebKitCookieManager *cookie_manager = webkit_network_session_get_cookie_manager(launcher->network_session);
+#else
+    WebKitCookieManager *cookie_manager = webkit_website_data_manager_get_cookie_manager(launcher->web_data_manager);
+#endif
+    webkit_cookie_manager_add_cookie(cookie_manager,
                                      cookie,
                                      NULL, // GCancellable
                                      (GAsyncReadyCallback) on_cookie_added,
@@ -903,7 +997,7 @@ cog_launcher_add_mem_pressure_option_entries(CogLauncher *self)
                            "  and thresholds relative to the limit which determine at which points memory\n"
                            "  will be reclaimed. The conservative threshold is typically lower when reached\n"
                            "  memory will be reclaimed; the strict threshold works in the same way but the\n"
-                           "  process is more aggresive. The kill threshold configures when worker processes\n"
+                           "  process is more aggressive. The kill threshold configures when worker processes\n"
                            "  will be forcibly killed. Note that if there is no memory limit set, the other\n"
                            "  settings are ignored.\n",
                            "Options to configure memory usage limits",
@@ -958,19 +1052,82 @@ cog_launcher_add_web_permissions_option_entries(CogLauncher *launcher)
     g_application_add_option_group(G_APPLICATION(launcher), g_steal_pointer(&option_group));
 }
 
+#if WEBKIT_CHECK_VERSION(2, 42, 0)
+static WebKitFeature *
+find_feature(WebKitFeatureList *features, const char *identifier)
+{
+    for (gsize i = 0; i < webkit_feature_list_get_length(features); i++) {
+        WebKitFeature *feature = webkit_feature_list_get(features, i);
+        if (!g_ascii_strcasecmp(identifier, webkit_feature_get_identifier(feature)))
+            return feature;
+    }
+    return NULL;
+}
+
+static gboolean
+cog_features_option_callback(const gchar *option, const gchar *value, WebKitSettings *web_settings, GError **error)
+{
+    g_autoptr(WebKitFeatureList) features = webkit_settings_get_all_features();
+
+    if (!strcmp(value, "help")) {
+        g_print("Multiple feature names may be specified separated by commas. No prefix or '+' enable\n"
+                "features, prefixes '-' and '!' disable features. Names are case-insensitive. Example:\n"
+                "\n    %s --features='!DirPseudo,+WebAnimationsCustomEffects,webgl'\n\n"
+                "Available features (+/- = enabled/disabled by default):\n\n",
+                g_get_prgname());
+        g_autoptr(GEnumClass) statusEnum = g_type_class_ref(WEBKIT_TYPE_FEATURE_STATUS);
+        for (gsize i = 0; i < webkit_feature_list_get_length(features); i++) {
+            WebKitFeature *feature = webkit_feature_list_get(features, i);
+            g_print("  %c %s (%s)",
+                    webkit_feature_get_default_value(feature) ? '+' : '-',
+                    webkit_feature_get_identifier(feature),
+                    g_enum_get_value(statusEnum, webkit_feature_get_status(feature))->value_nick);
+            if (webkit_feature_get_name(feature))
+                g_print(": %s", webkit_feature_get_name(feature));
+            g_print("\n");
+        }
+        exit(EXIT_SUCCESS);
+    }
+
+    g_auto(GStrv) items = g_strsplit(value, ",", -1);
+    for (gsize i = 0; items[i]; i++) {
+        char    *item = g_strchomp(items[i]);
+        gboolean enabled = TRUE;
+        switch (item[0]) {
+        case '!':
+        case '-':
+            enabled = FALSE;
+            G_GNUC_FALLTHROUGH;
+        case '+':
+            item++;
+            G_GNUC_FALLTHROUGH;
+        default:
+            break;
+        }
+
+        if (item[0] == '\0') {
+            g_set_error_literal(error, G_OPTION_ERROR, G_OPTION_ERROR_FAILED, "Empty feature name specified");
+            return FALSE;
+        }
+
+        WebKitFeature *feature = find_feature(features, item);
+        if (!feature) {
+            g_set_error(error, G_OPTION_ERROR, G_OPTION_ERROR_FAILED, "Feature '%s' is not available", item);
+            return FALSE;
+        }
+
+        webkit_settings_set_feature_enabled(web_settings, feature, enabled);
+    }
+
+    return TRUE;
+}
+#endif /* WEBKIT_CHECK_VERSION */
+
 static void
 cog_launcher_add_web_settings_option_entries(CogLauncher *launcher)
 {
     g_return_if_fail(COG_IS_LAUNCHER(launcher));
 
-    g_autofree GOptionEntry *option_entries = cog_option_entries_from_class(G_OBJECT_GET_CLASS(launcher->web_settings));
-
-    if (!option_entries) {
-        g_critical("Could not deduce option entries for WebKitSettings."
-                   " This should not happen, continuing but YMMV.");
-        return;
-    }
-
     g_autoptr(GOptionGroup) option_group = g_option_group_new(
         "websettings",
         "WebKitSettings options can be used to configure features exposed to the loaded Web content.\n"
@@ -983,7 +1140,31 @@ cog_launcher_add_web_settings_option_entries(CogLauncher *launcher)
         "Show WebKitSettings options",
         launcher->web_settings,
         NULL);
-    g_option_group_add_entries(option_group, option_entries);
+
+    g_autofree GOptionEntry *option_entries = cog_option_entries_from_class(G_OBJECT_GET_CLASS(launcher->web_settings));
+    if (option_entries) {
+        g_option_group_add_entries(option_group, option_entries);
+    } else {
+        g_critical("Could not deduce option entries for WebKitSettings."
+                   " This should not happen, continuing but YMMV.");
+    }
+
+#if WEBKIT_CHECK_VERSION(2, 42, 0)
+    static const GOptionEntry features_entries[] = {
+        {
+            .short_name = 'F',
+            .long_name = "features",
+            .description = "Enable or disable WebKit features (hint: pass 'help' for a list)",
+            .arg = G_OPTION_ARG_CALLBACK,
+            .arg_data = cog_features_option_callback,
+            .arg_description = "FEATURE-LIST",
+        },
+        {
+            NULL,
+        }};
+    g_option_group_add_entries(option_group, features_entries);
+#endif /* WEBKIT_CHECK_VERSION */
+
     g_application_add_option_group(G_APPLICATION(launcher), g_steal_pointer(&option_group));
 }
 
@@ -1025,8 +1206,10 @@ static GOptionEntry s_cli_options[] = {
      "Ignore TLS errors (default: disabled).", NULL},
     {"content-filter", 'F', 0, G_OPTION_ARG_FILENAME, &s_options.filter_path,
      "Path to content filter JSON rule set (default: none).", "PATH"},
+#if !COG_USE_WPE2
     {"enable-sandbox", 's', 0, G_OPTION_ARG_NONE, &s_options.enable_sandbox,
      "Enable WebProcess sandbox (default: disabled).", NULL},
+#endif
     {"automation", '\0', 0, G_OPTION_ARG_NONE, &s_options.automation, "Enable automation mode (default: disabled).",
      NULL},
 #if HAVE_WEBKIT_NETWORK_PROXY_API
@@ -1044,11 +1227,17 @@ cog_launcher_constructed(GObject *object)
 
     CogLauncher *launcher = COG_LAUNCHER(object);
 
+#if COG_USE_WPE2
+    if (!launcher->automated)
+        launcher->network_session = webkit_network_session_new(NULL, NULL);
+#else
+
     g_autofree char *data_dir = g_build_filename(g_get_user_data_dir(), g_get_prgname(), NULL);
     g_autofree char *cache_dir = g_build_filename(g_get_user_cache_dir(), g_get_prgname(), NULL);
 
     launcher->web_data_manager = g_object_new(WEBKIT_TYPE_WEBSITE_DATA_MANAGER, "is-ephemeral", launcher->automated,
                                               "base-data-directory", data_dir, "base-cache-directory", cache_dir, NULL);
+#endif
 
     cog_launcher_add_action(launcher, "quit", on_action_quit, NULL);
     cog_launcher_add_action(launcher, "previous", on_action_prev, NULL);
@@ -1180,8 +1369,7 @@ cog_launcher_handle_local_options(GApplication *application, GVariantDict *optio
         g_printerr("%s: URI '%s' is invalid UTF-8: %s\n", g_get_prgname(), uri, error->message);
         return EXIT_FAILURE;
     }
-    g_strfreev(s_options.arguments);
-    s_options.arguments = NULL;
+    g_clear_pointer(&s_options.arguments, g_strfreev);
 
     /*
      * Validate the supplied local URI handler specification and check
@@ -1217,8 +1405,7 @@ cog_launcher_handle_local_options(GApplication *application, GVariantDict *optio
         *colon = '\0'; /* NULL-terminate the URI scheme name. */
         g_hash_table_insert(handler_map, g_strdup(s_options.dir_handlers[i]), cog_directory_files_handler_new(file));
     }
-    if (s_options.dir_handlers)
-        g_strfreev(s_options.dir_handlers);
+    g_clear_pointer(&s_options.dir_handlers, g_strfreev);
     s_options.handler_map = g_hash_table_size(handler_map) ? g_steal_pointer(&handler_map) : NULL;
 
     s_options.home_uri = g_steal_pointer(&utf8_uri);
@@ -1240,14 +1427,12 @@ cog_launcher_handle_local_options(GApplication *application, GVariantDict *optio
             return EXIT_FAILURE;
         }
 
-        g_free(s_options.config_file);
+        g_clear_pointer(&s_options.config_file, g_free);
         s_options.key_file = g_steal_pointer(&key_file);
     }
 
     if (s_options.filter_path) {
-        WebKitWebsiteDataManager *data_manager = cog_launcher_get_web_data_manager(launcher);
-        g_autofree char          *filters_path =
-            g_build_filename(webkit_website_data_manager_get_base_cache_directory(data_manager), "filters", NULL);
+        g_autofree char *filters_path = g_build_filename(g_get_user_cache_dir(), g_get_prgname(), "filters", NULL);
         g_autoptr(WebKitUserContentFilterStore) store = webkit_user_content_filter_store_new(filters_path);
 
         g_autoptr(GFile) file = g_file_new_for_commandline_arg(s_options.filter_path);
@@ -1265,11 +1450,24 @@ cog_launcher_handle_local_options(GApplication *application, GVariantDict *optio
 
 #if HAVE_WEBKIT_NETWORK_PROXY_API
     if (s_options.proxy) {
-        WebKitWebsiteDataManager             *data_manager = cog_launcher_get_web_data_manager(launcher);
+        if (launcher->automated) {
+            g_printerr("%s: Cannot set proxy settings in automation mode\n", g_get_prgname());
+            return EXIT_FAILURE;
+        }
+
         g_autoptr(WebKitNetworkProxySettings) webkit_proxy_settings =
             webkit_network_proxy_settings_new(s_options.proxy, (const gchar *const *) s_options.ignore_hosts);
-        webkit_website_data_manager_set_network_proxy_settings(data_manager, WEBKIT_NETWORK_PROXY_MODE_CUSTOM,
+        g_clear_pointer(&s_options.proxy, g_free);
+        g_clear_pointer(&s_options.ignore_hosts, g_strfreev);
+#    if COG_USE_WPE2
+        webkit_network_session_set_proxy_settings(launcher->network_session,
+                                                  WEBKIT_NETWORK_PROXY_MODE_CUSTOM,
+                                                  webkit_proxy_settings);
+#    else
+        webkit_website_data_manager_set_network_proxy_settings(launcher->web_data_manager,
+                                                               WEBKIT_NETWORK_PROXY_MODE_CUSTOM,
                                                                webkit_proxy_settings);
+#    endif
     }
 #endif /* HAVE_WEBKIT_NETWORK_PROXY_API */
 
@@ -1303,6 +1501,7 @@ cog_launcher_class_init(CogLauncherClass *klass)
     application_class->startup = cog_launcher_startup;
     application_class->shutdown = cog_launcher_shutdown;
     application_class->handle_local_options = cog_launcher_handle_local_options;
+    application_class->activate = cog_launcher_activate;
 
     g_object_class_install_property(object_class,
                                     PROP_AUTOMATED,
@@ -1376,5 +1575,10 @@ WebKitWebsiteDataManager *
 cog_launcher_get_web_data_manager(CogLauncher *launcher)
 {
     g_return_val_if_fail(COG_IS_LAUNCHER(launcher), NULL);
+#if COG_USE_WPE2
+    return launcher->network_session ? webkit_network_session_get_website_data_manager(launcher->network_session)
+                                     : NULL;
+#else
     return launcher->web_data_manager;
+#endif
 }
