@@ -351,6 +351,21 @@ check_drm(void)
     return supported;
 }
 
+static int32_t
+find_crtc_for_encoder(const drmModeRes *resources, const drmModeEncoder *encoder)
+{
+    for (int i = 0; i < resources->count_crtcs; i++) {
+        const uint32_t crtc_mask = 1 << i;
+        const uint32_t crtc_id = resources->crtcs[i];
+        if (encoder->possible_crtcs & crtc_mask && encoder->crtc_id == crtc_id) {
+            return crtc_id;
+        }
+    }
+
+    /* no match found */
+    return -1;
+}
+
 static gboolean
 init_drm(void)
 {
@@ -490,22 +505,33 @@ init_drm(void)
         return FALSE;
 
     g_debug("init_drm: using mode [%ld] '%s' @ %dHz",
-            (drm_data.mode - drm_data.connector.obj->modes) / sizeof(drmModeModeInfo *), drm_data.mode->name,
+            (long)((drm_data.mode - drm_data.connector.obj->modes) / sizeof(drmModeModeInfo *)), drm_data.mode->name,
             drm_data.mode->vrefresh);
 
+    /* Try the currently connected encoder+crtc */
     for (int i = 0; i < drm_data.base_resources->count_encoders; ++i) {
         drm_data.encoder = drmModeGetEncoder(drm_data.fd, drm_data.base_resources->encoders[i]);
-        if (drm_data.encoder->encoder_id == drm_data.connector.obj->encoder_id)
+        if (!drm_data.encoder) {
+            /* cannot retrieve encoder, ignoring... */
+            continue;
+        }
+
+        const int32_t crtc_id = find_crtc_for_encoder(drm_data.base_resources, drm_data.encoder);
+        if (crtc_id != -1) {
+            drm_data.crtc.obj_id = crtc_id;
             break;
+        }
 
         g_clear_pointer (&drm_data.encoder, drmModeFreeEncoder);
     }
-    if (!drm_data.encoder)
+
+    if (!drm_data.encoder) {
+        fprintf(stderr, "no crtc for encoder found!\n");
         return FALSE;
+    }
 
     drm_data.connector.obj_id = drm_data.connector.obj->connector_id;
 
-    drm_data.crtc.obj_id = drm_data.encoder->crtc_id;
     drm_data.crtc.obj = drmModeGetCrtc (drm_data.fd, drm_data.crtc.obj_id);
     for (int i = 0; i < drm_data.base_resources->count_crtcs; ++i) {
         if (drm_data.base_resources->crtcs[i] == drm_data.crtc.obj_id) {
@@ -689,8 +715,25 @@ static void
 input_dispatch_key_event (uint32_t time, uint32_t key, enum libinput_key_state state)
 {
     struct wpe_input_xkb_context *default_context = wpe_input_xkb_context_get_default ();
+
+    // if wpe is unable to prepare an xkb context (e.g. environment without
+    // any prepared keymap data), ignore the key event
+    if (!default_context)
+        return;
+
+    // libwpe (<=v1.14.1) may provide a broken context; ensure an underlying
+    // xkb_context has been configured
+    struct xkb_context *ctx = wpe_input_xkb_context_get_context(default_context);
+    if (!ctx)
+        return;
+
     struct xkb_state *context_state = wpe_input_xkb_context_get_state (default_context);
 
+    // if wpe cannot determine the xkb state (e.g. required keymap does are not
+    // available in this environment), ignore the key event
+    if (!context_state)
+        return;
+
     uint32_t keysym = wpe_input_xkb_context_get_key_code (default_context, key, !!state);
 
     xkb_state_update_key (context_state, key, !!state ? XKB_KEY_DOWN : XKB_KEY_UP);
@@ -1178,9 +1221,28 @@ clear_input(CogDrmPlatform *platform)
     g_clear_pointer (&input_data.udev, udev_unref);
 }
 
+static void
+update_logical_input_size(CogGLRendererRotation rotation)
+{
+    switch (rotation) {
+    case COG_GL_RENDERER_ROTATION_0:
+    case COG_GL_RENDERER_ROTATION_180:
+        input_data.input_width = drm_data.mode->hdisplay;
+        input_data.input_height = drm_data.mode->vdisplay;
+        break;
+    case COG_GL_RENDERER_ROTATION_90:
+    case COG_GL_RENDERER_ROTATION_270:
+        input_data.input_width = drm_data.mode->vdisplay;
+        input_data.input_height = drm_data.mode->hdisplay;
+        break;
+    }
+}
+
 static gboolean
 init_input(CogDrmPlatform *platform)
 {
+    update_logical_input_size(platform->rotation);
+
     static struct libinput_interface interface = {
         .open_restricted = input_interface_open_restricted,
         .close_restricted = input_interface_close_restricted,
@@ -1198,9 +1260,6 @@ init_input(CogDrmPlatform *platform)
     if (ret)
         return FALSE;
 
-    input_data.input_width = drm_data.mode->hdisplay;
-    input_data.input_height = drm_data.mode->vdisplay;
-
     for (int i = 0; i < G_N_ELEMENTS (input_data.touch_points); ++i) {
         struct wpe_input_touch_event_raw *touch_point = &input_data.touch_points[i];
 
@@ -1258,6 +1317,12 @@ key_repeat_source_dispatch (GSource *base, GSourceFunc callback, gpointer user_d
 }
 
 
+static void
+cog_drm_platform_shell_device_factor_changed(CogShell *shell, GParamSpec *param_spec, gpointer data)
+{
+    drm_data.device_scale = cog_shell_get_device_scale_factor(shell);
+}
+
 static void
 clear_glib (void)
 {
@@ -1342,6 +1407,7 @@ cog_drm_platform_setup(CogPlatform *platform, CogShell *shell, const char *param
     CogDrmPlatform *self = COG_DRM_PLATFORM(platform);
 
     init_config(COG_DRM_PLATFORM(platform), shell, params);
+    g_signal_connect(shell, "notify::device-scale-factor", G_CALLBACK(cog_drm_platform_shell_device_factor_changed), NULL);
 
     if (!wpe_loader_init ("libWPEBackend-fdo-1.0.so")) {
         g_set_error_literal (error,
@@ -1504,9 +1570,9 @@ cog_drm_platform_set_property(GObject *object, unsigned prop_id, const GValue *v
             return;
 
         if (!self->renderer) {
-            self->rotation = rotation;
+            update_logical_input_size(self->rotation = rotation);
         } else if (cog_drm_renderer_set_rotation(self->renderer, rotation)) {
-            self->rotation = rotation;
+            update_logical_input_size(self->rotation = rotation);
             if (self->rotatable_input_devices)
                 g_list_foreach(self->rotatable_input_devices, (GFunc) input_configure_device, self);
         } else {
